@@ -14,7 +14,7 @@
     You should have received a copy of the GNU Affero General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
-#include "../config.h"
+#include "config.h"
 #if defined(__ANDROID__) || defined(_WIN32)
 #include <stdint.h>
 #include <stddef.h>
@@ -26,8 +26,16 @@
 #ifdef NO_STRNDUP
   extern char* strndup(const char* in, unsigned int length);
 #endif
+#ifdef POLL_BROKEN_OR_MISSING
+  #include <glib.h>
+  #define poll g_poll
+  #define pollfd _GPollFD
+  #define POLLIN G_IO_IN
+#endif
+#ifdef NO_PRCTL
+  #define prctl(x,y)
+#endif
 #ifdef _WIN32
-  #define prctl(...)
   #define wait(x)
   #define mkdir(x,y) mkdir(x)
 #endif
