@@ -23,7 +23,6 @@
   #include <wtypes.h>
   #include <winbase.h>
 #else
-  #include <sys/prctl.h>
   #include <sys/wait.h>
 #endif
 #include <ctype.h>
@@ -46,17 +45,22 @@
 #endif
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include "../compat.h"
+#include "../compat_av.h"
+#ifndef NO_PRCTL
+  #include <sys/prctl.h>
+#endif
 #include "../libcamera/camera.h"
 #include "userlist.h"
 #include "media.h"
 #include "compat.h"
-#include "config.h"
+#include "configfile.h"
 #include "gui.h"
 #include "logging.h"
 #include "../stringutils.h"
-#include "../compat.h"
-#include "../compat_av.h"
 #include "inputhistory.h"
+#include "playmedia.h"
+#include "greenroom.h"
 #include "main.h"
 
 struct viddata
@@ -72,10 +76,11 @@ int tc_client_in[2];
 const char* channel=0;
 const char* mycolor=0;
 char* nickname=0;
+char hasgreenroom=0;
 char frombuild=0; // Running from the build directory
-#define TC_CLIENT (frombuild?"./tc_client":"tc_client")
 #ifdef _WIN32
   PROCESS_INFORMATION coreprocess={.hProcess=0};
+  PROCESS_INFORMATION grprocess={.hProcess=0}; // Greenroom
 #endif
 
 void printchat(const char* text, const char* color, unsigned int offset, const char* pm)
@@ -140,9 +145,9 @@ void printchat(const char* text, const char* color, unsigned int offset, const c
 }
 
 unsigned int cameventsource=0;
-char buf[1024];
 gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
 {
+  static char buf[1024];
   gsize r;
   unsigned int i;
   for(i=0; i<1023; ++i)
@@ -157,8 +162,6 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     char* sizestr=strchr(&buf[7], ' ');
     if(!sizestr){return 1;}
     sizestr[0]=0;
-    // Find the camera representation for the given ID
-    struct camera* cam=camera_find(&buf[7]);
     unsigned int size=strtoul(&sizestr[1], 0, 0);
     if(!size){return 1;}
     // Mostly ignore the first byte (contains frame type (e.g. keyframe etc.) in 4 bits and codec in the other 4)
@@ -167,6 +170,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     av_init_packet(&pkt);
     unsigned char databuf[size+4];
     pkt.data=databuf;
+    pkt.size=size;
     unsigned char frameinfo;
     g_io_channel_read_chars(iochannel, (gchar*)&frameinfo, 1, 0, 0);
 //   printf("Frametype-frame: %x\n", ((unsigned int)frameinfo&0xf0)/16);
@@ -178,32 +182,10 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
       pos+=r;
     }
     if((frameinfo&0xf)!=2){return 1;} // Not FLV1, get data but discard it
+    // Find the camera representation for the given ID
+    struct camera* cam=camera_find(&buf[7]);
     if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
-    pkt.size=size;
-    int gotframe;
-    avcodec_send_packet(cam->vctx, &pkt);
-    gotframe=avcodec_receive_frame(cam->vctx, cam->frame);
-    if(gotframe){return 1;}
-
-    if(cam->placeholder) // Remove the placeholder animation if it has it
-    {
-      g_source_remove(cam->placeholder);
-      cam->placeholder=0;
-    }
-    // Scale and convert to RGB24 format
-    unsigned int bufsize=av_image_get_buffer_size(AV_PIX_FMT_RGB24, camsize_scale.width, camsize_scale.height, 1);
-    unsigned char* buf=malloc(bufsize);
-    cam->dstframe->data[0]=buf;
-    cam->dstframe->linesize[0]=camsize_scale.width*3;
-    struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, camsize_scale.width, camsize_scale.height, AV_PIX_FMT_RGB24, SWS_BICUBIC, 0, 0, 0);
-    sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
-    sws_freeContext(swsctx);
-    postprocess(&cam->postproc, cam->dstframe->data[0], camsize_scale.width, camsize_scale.height);
-
-    GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
-    GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->dstframe->data[0], GDK_COLORSPACE_RGB, 0, 8, camsize_scale.width, camsize_scale.height, cam->dstframe->linesize[0], freebuffer, 0);
-    gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
-    g_object_unref(oldpixbuf);
+    camera_decode(cam, &pkt, camsize_scale.width, camsize_scale.height);
     return 1;
   }
   if(!strncmp(buf, "Audio: ", 7))
@@ -232,22 +214,29 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     struct camera* cam=camera_find(&buf[7]);
     if(!cam){printf("No cam found with ID '%s'\n", &buf[7]); return 1;}
     if(!cam->actx && camera_init_audio(cam, frameinfo)){return 1;}
-    int gotframe;
-    avcodec_send_packet(cam->actx, &pkt);
-    gotframe=avcodec_receive_frame(cam->actx, cam->frame);
-    if(gotframe){return 1;}
-    unsigned int samplecount=cam->frame->nb_samples*SAMPLERATE_OUT/cam->samplerate;
-    int16_t outbuf[samplecount];
-    void* outdata[]={outbuf, 0};
-  #ifdef HAVE_AVRESAMPLE
-    int outlen=avresample_convert(cam->resamplectx, (void*)outdata, samplecount*sizeof(uint8_t), samplecount, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
-  #else
-    int outlen=swr_convert(cam->swrctx, (void*)outdata, samplecount, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
-  #endif
-    if(outlen>0){camera_playsnd(data->audiopipe, cam, outbuf, outlen);}
+    mic_decode(cam, &pkt);
 #endif
     return 1;
   }
+  if(!strncmp(buf, "Nickname of connection ", 23))
+  {
+    char* nick=strstr(&buf[23], ": ");
+    if(nick)
+    {
+      nick[0]=0;
+      if(greenroom_gotnick(&buf[23], &nick[2]))
+      {
+        char buf[strlen(&nick[2])+strlen(" is waiting in the greenroom0")];
+        strcpy(buf, &nick[2]);
+        strcat(buf, " is waiting in the greenroom");
+        printchat(buf, 0, 0, 0);
+      }else{
+        nick[0]=':';
+        printchat(buf, 0, 0, 0);
+      }
+    }
+    return 1;
+  }
   if(!strncmp(buf, "Currently online: ", 18))
   {
     printchat(buf, 0, 0, 0);
@@ -263,10 +252,12 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
   }
   if(!strncmp(buf, "Connection ID: ", 15)) // Our initial nickname is "guest-" plus our connection ID
   {
+    if(config_get_bool("disablesnapshots")){write(tc_client_in[1], "/disablesnapshots\n", 18);}
     write(tc_client_in[1], "/color\n", 7); // Check which random color tc_client picked
     unsigned int length=strlen(&buf[15]);
     nickname=malloc(length+strlen("guest-")+1);
     sprintf(nickname, "guest-%s", &(buf[15]));
+    if(hasgreenroom){greenroom_join(&buf[15]);}
     return 1;
   }
   if(!strncmp(buf, "Captcha: ", 9))
@@ -294,6 +285,11 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     }
     return 1;
   }
+  if(!strcmp(buf, "Authenticated to broadcast"))
+  {
+    greenroom_allowed();
+    return 1;
+  }
   if(!strcmp(buf, "Password required"))
   {
     gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
@@ -337,6 +333,10 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     if(!space){return 1;}
     if(space[-1]==':')
     {
+      char nickbuf[space-nick];
+      memcpy(nickbuf, nick, &space[-1]-nick);
+      nickbuf[&space[-1]-nick]=0;
+      struct user* user=finduser(nickbuf);
       if(config_get_bool("soundradio_cmd"))
       {
 #ifdef _WIN32
@@ -351,48 +351,61 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
         }
 #endif
       }
-      if(!strncmp(space, " /mbs youTube ", 14) && config_get_bool("youtuberadio_cmd"))
+      if(user && user->ismod && !strncmp(space, " /mbs youTube ", 14))
       {
-#ifndef _WIN32
-        if(!fork())
-#else
-        char* spacetmp=space;
-        space=strdup(space);
-#endif
+        if(config_get_bool("youtuberadio_cmd"))
         {
-// TODO: store the PID and make sure it's dead before starting a new video? and upon /mbc?
-// TODO: only play videos from mods?
-          char* id=&space[14];
-          char* offset=strchr(id, ' ');
-          if(!offset){_exit(1);}
-          offset[0]=0;
-          offset=&offset[1];
-          char* end=strchr(offset, ' '); // Ignore any additional arguments after the offset (the modbot utility includes the video title here)
-          if(end){end[0]=0;}
-          // Handle format string
-          const char* fmt=config_get_str("youtubecmd");
-          int len=strlen(fmt)+1;
-          len+=strcount(fmt, "%i")*(strlen(id)-2);
-          len+=strcount(fmt, "%t")*(strlen(id)-2);
-          char cmd[len];
-          cmd[0]=0;
-          while(fmt[0])
+          #ifndef _WIN32
+          if(!fork())
+          #else
+          char* spacetmp=space;
+          space=strdup(space);
+          #endif
           {
-            if(!strncmp(fmt, "%i", 2)){strcat(cmd, id); fmt=&fmt[2]; continue;}
-            if(!strncmp(fmt, "%t", 2)){strcat(cmd, offset); fmt=&fmt[2]; continue;}
-            for(len=0; fmt[len] && strncmp(&fmt[len], "%i", 2) && strncmp(&fmt[len], "%t", 2); ++len);
-            strncat(cmd, fmt, len);
-            fmt=&fmt[len];
+// TODO: store the PID and make sure it's dead before starting a new video? and upon /mbc?
+            char* id=&space[14];
+            char* offset=strchr(id, ' ');
+            if(!offset){_exit(1);}
+            offset[0]=0;
+            offset=&offset[1];
+            char* end=strchr(offset, ' '); // Ignore any additional arguments after the offset (the modbot utility includes the video title here)
+            if(end){end[0]=0;}
+            // Handle format string
+            const char* fmt=config_get_str("youtubecmd");
+            int len=strlen(fmt)+1;
+            len+=strcount(fmt, "%i")*(strlen(id)-2);
+            len+=strcount(fmt, "%t")*(strlen(id)-2);
+            char cmd[len];
+            cmd[0]=0;
+            while(fmt[0])
+            {
+              if(!strncmp(fmt, "%i", 2)){strcat(cmd, id); fmt=&fmt[2]; continue;}
+              if(!strncmp(fmt, "%t", 2)){strcat(cmd, offset); fmt=&fmt[2]; continue;}
+              for(len=0; fmt[len] && strncmp(&fmt[len], "%i", 2) && strncmp(&fmt[len], "%t", 2); ++len);
+              strncat(cmd, fmt, len);
+              fmt=&fmt[len];
+            }
+            #ifdef _WIN32
+            w32_runcmd(cmd);
+            free(space);
+            space=spacetmp;
+            #else
+            execlp("sh", "sh", "-c", cmd, (char*)0);
+            _exit(0);
+            #endif
           }
-#ifdef _WIN32
-          w32_runcmd(cmd);
-          free(space);
-          space=spacetmp;
-#else
-          execlp("sh", "sh", "-c", cmd, (char*)0);
-          _exit(0);
-#endif
         }
+#ifdef HAVE_AVFORMAT
+        else if(config_get_bool("youtuberadio_embed")){media_play(&space[6]);}
+      }
+      else if(user && user->ismod && !strncmp(space, " /mbsk youTube ", 15) && config_get_bool("youtuberadio_embed"))
+      {
+        media_seek(strtol(&space[15], 0, 0));
+      }
+      else if(user && user->ismod && !strncmp(space, " /mbc ", 6) && config_get_bool("youtuberadio_embed"))
+      {
+        media_close();
+#endif
       }
       // Handle incoming PMs
       else if(!strncmp(space, " /msg ", 6))
@@ -410,7 +423,10 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     }
     if(config_get_bool("enable_logging")){logger_write(buf, channel, pm);}
     // Insert new content
-    printchat(buf, color, 8, pm);
+    if(space[-1]==':' || !config_get_bool("hide_notifications"))
+    {
+      printchat(buf, color, 8, pm);
+    }
     pm_highlight(pm);
     free(pm);
     if(space[-1]!=':') // Not a message
@@ -447,6 +463,7 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
           free(nickname);
           nickname=strdup(&space[21]);
         }
+        greenroom_changenick(nick, &space[21]);
       }
     }
     free(color);
@@ -467,19 +484,13 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
   if(space && !strcmp(space, " is a moderator."))
   {
     space[0]=0;
-    struct user* user=finduser(buf);
-    if(user)
-    {
-      user->ismod=1;
-      renameuser(buf, buf); // Update the userlist label
-    }
+    usersetmod(buf, 1);
     return 1;
   }
   if(space && !strcmp(space, " is no longer a moderator."))
   {
     space[0]=0;
-    struct user* user=finduser(buf);
-    if(user){user->ismod=0;}
+    usersetmod(buf, 0);
     return 1;
   }
   // Start a stream when someone cams up
@@ -504,16 +515,15 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     if(!idend){return 1;}
     idend[0]=0;
     camera_remove(nick, 1); // Remove any duplicates
-    struct camera* cam=camera_new(nick, id);
-    cam->vctx=avcodec_alloc_context3(data->vdecoder);
-    avcodec_open2(cam->vctx, data->vdecoder, 0);
+    struct camera* cam=camera_new(nick, id, CAMFLAG_NONE);
     updatescaling(0, 0, 1);
     gtk_widget_show_all(cam->box);
     return 1;
   }
   if(!strcmp(buf, "Starting outgoing media stream"))
   {
-    struct camera* cam=camera_new(nickname, "out");
+    if(camera_find("out")){return 1;} // In case we're just moving out of greenroom
+    struct camera* cam=camera_new(nickname, "out", CAMFLAG_NONE);
     cam->vctx=avcodec_alloc_context3(data->vencoder);
     cam->vctx->pix_fmt=AV_PIX_FMT_YUV420P;
     cam->vctx->time_base.num=1;
@@ -547,6 +557,17 @@ gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x)
     printchat(buf, 0, 0, 0);
     return 1;
   }
+  if(!strncmp(buf, "Captcha not completed,", 22))
+  {
+    printchat(buf, 0, 0, 0);
+    gui_disableinputs();
+    return 1;
+  }
+  if(!strcmp(buf, "Channel has greenroom"))
+  {
+    hasgreenroom=1;
+    return 1;
+  }
   if(!strcmp(buf, "Server disconnected"))
   {
     printchat(buf, 0, 0, 0);
@@ -597,7 +618,8 @@ void togglecam(GtkCheckMenuItem* item, void* x)
     // TODO: if switching from cam+mic to just mic, send something to tell other clients to drop the last video frame and show a mic instead
     if(camera_find("out"))
     {
-      dprintf(tc_client_in[1], "/camdown\n");
+      int fd=((hasgreenroom && !greenroom_gotpass)?greenroompipe_in[1]:tc_client_in[1]);
+      dprintf(fd, "/camdown\n");
       camera_remove("out", 0); // Close our local display
     }
     return;
@@ -610,6 +632,7 @@ void togglemic(GtkCheckMenuItem* item, void* x)
 {
   static int micpipe[2];
   static int eventsource=0;
+  int fd=((hasgreenroom && !greenroom_gotpass)?greenroompipe_in[1]:tc_client_in[1]);
   if(!gtk_check_menu_item_get_active(item)) // Stop mic broadcast
   {
     gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
@@ -622,7 +645,7 @@ void togglemic(GtkCheckMenuItem* item, void* x)
     if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera")))){return;}
     if(camera_find("out"))
     {
-      dprintf(tc_client_in[1], "/camdown\n");
+      dprintf(fd, "/camdown\n");
       camera_remove("out", 0); // Close our local display
     }
     return;
@@ -644,7 +667,7 @@ void togglemic(GtkCheckMenuItem* item, void* x)
   // Start mic broadcast
   if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_camera"))))
   { // Only /camup if we're not already broadcasting video
-    dprintf(tc_client_in[1], "/camup\n");
+    dprintf(fd, "/camup\n");
   }
   pipe(micpipe);
   g_thread_new("audio_in", audiothread_in, &micpipe[1]);
@@ -750,6 +773,20 @@ void sendmessage(GtkEntry* entry, void* x)
     sendingmsg=0;
     return;
   }
+#ifdef HAVE_AVFORMAT
+  else if(!strncmp(msg, "/mbs ", 5) && config_get_bool("youtuberadio_embed"))
+  {
+    media_play(&msg[5]);
+  }
+  else if(!strncmp(msg, "/mbsk ", 6) && config_get_bool("youtuberadio_embed"))
+  {
+    media_seek(strtol(&msg[6], 0, 0));
+  }
+  else if(!strncmp(msg, "/mbc ", 5) && config_get_bool("youtuberadio_embed"))
+  {
+    media_close();
+  }
+#endif
   else if(msg[0]!='/') // If we're in a PM tab, send messages as PMs
   {
     GtkNotebook* tabs=GTK_NOTEBOOK(gtk_builder_get_object(gui, "tabs"));
@@ -787,7 +824,8 @@ void sendmessage(GtkEntry* entry, void* x)
      !strcmp(msg, "/camup") ||
      !strcmp(msg, "/camdown") ||
      !strncmp(msg, "/video ", 7) ||
-     !strncmp(msg, "/topic ", 7))
+     !strncmp(msg, "/topic ", 7) ||
+     !strncmp(msg, "/getnick ", 9))
   {
     gtk_entry_set_text(entry, "");
     sendingmsg=0;
@@ -855,8 +893,7 @@ void startsession(GtkButton* button, void* x)
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "startwindow")));
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelconfig")));
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "channelpasswordwindow")));
-  gtk_widget_show_all(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
-  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
+  GtkWidget* mainwindow=GTK_WIDGET(gtk_builder_get_object(gui, "main"));
   const char* nick=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_nick")));
   channel=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "cc_channel")));
   const char* chanpass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "channelpassword")));
@@ -864,6 +901,11 @@ void startsession(GtkButton* button, void* x)
   const char* acc_user=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_username")));
   const char* acc_pass=gtk_entry_get_text(GTK_ENTRY(gtk_builder_get_object(gui, "acc_password")));
   const char* cookiename=(acc_user[0]?acc_user:"no_account");
+  char title[strlen("tc_client ()0")+strlen(channel)];
+  sprintf(title, "tc_client (%s)", channel);
+  gtk_window_set_title(GTK_WINDOW(mainwindow), title);
+  gtk_widget_show_all(mainwindow);
+  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "pushtotalk")));
 #ifdef _WIN32
   char cmd[strlen("./tc_client --hexcolors --cookies tinychat_.cookie -u    0")+strlen(cookiename)+strlen(acc_user)+strlen(channel)+strlen(nick)+strlen(chanpass)];
   strcpy(cmd, "./tc_client --hexcolors ");
@@ -933,8 +975,13 @@ void captcha_done(GtkWidget* button, void* x)
   gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(gui, "captcha")));
   gtk_widget_show(GTK_WIDGET(gtk_builder_get_object(gui, "main")));
   write(tc_client_in[1], "\n", 1);
+  if(greenroompipe_in[1]>-1){write(greenroompipe_in[1], "\n", 1);}
 }
 
+#ifndef _WIN32
+void justwait(int x){waitpid(-1, 0, WNOHANG);}
+#endif
+
 int main(int argc, char** argv)
 {
   if(!strncmp(argv[0], "./", 2)){frombuild=1;}
@@ -944,7 +991,7 @@ int main(int argc, char** argv)
   data->vdecoder=avcodec_find_decoder(AV_CODEC_ID_FLV1);
   data->vencoder=avcodec_find_encoder(AV_CODEC_ID_FLV1);
 #ifndef _WIN32
-  signal(SIGCHLD, SIG_IGN);
+  signal(SIGCHLD, justwait); // SIG_IGN causes trouble with libpulse's autospawn
 #else
   frombuild=1;
 #endif
@@ -961,6 +1008,9 @@ int main(int argc, char** argv)
   gui_init(frombuild);
   campreview.frame=av_frame_alloc();
   campreview.frame->data[0]=0;
+#ifdef HAVE_LIBAO
+  g_timeout_add(40, audiomixer, &audiopipe[1]);
+#endif
   gtk_main();
  
 #ifdef _WIN32
@@ -968,6 +1018,10 @@ int main(int argc, char** argv)
   {
     TerminateProcess(coreprocess.hProcess, 0);
   }
+  if(grprocess.hProcess)
+  {
+    TerminateProcess(grprocess.hProcess, 0);
+  }
 #endif
   camera_cleanup();
   return 0;
