@@ -18,7 +18,7 @@
 #include <string.h>
 #include <stdint.h>
 #include <gtk/gtk.h>
-#include "config.h"
+#include "configfile.h"
 #include "logging.h"
 #include "compat.h"
 #include "userlist.h"
@@ -35,12 +35,40 @@ extern gboolean gui_greenscreen_preview(void* x);
 GdkCursor* gui_cursor_text;
 GdkCursor* gui_cursor_link;
 struct chatview* mainchat;
+GdkPixbuf* modicon=0;
 
 void settings_reset(GtkBuilder* gui)
 {
-  // Font
+  // Text
   GtkWidget* option=GTK_WIDGET(gtk_builder_get_object(gui, "fontsize"));
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(option), config_get_set("fontsize")?config_get_double("fontsize"):8);
+  // Find the matching combo box item
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "linewrap"));
+  GtkTreeModel* model=gtk_combo_box_get_model(GTK_COMBO_BOX(option));
+  GtkTreeIter iter;
+  if(config_get_set("linewrap_mode"))
+  {
+    char next=gtk_tree_model_get_iter_first(model, &iter);
+    while(next)
+    {
+      GtkWrapMode mode;
+      gtk_tree_model_get(model, &iter, 1, &mode, -1);
+      if(mode==config_get_int("linewrap_mode"))
+      {
+        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(option), &iter);
+        break;
+      }
+      next=gtk_tree_model_iter_next(model, &iter);
+    }
+  }
+  else if(gtk_tree_model_get_iter_first(model, &iter))
+  {
+    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(option), &iter);
+  }
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "bluelinks"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("blue_links"));
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "hidenotifications"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("hide_notifications"));
   // Sound
   option=GTK_WIDGET(gtk_builder_get_object(gui, "soundradio_cmd"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("soundradio_cmd"));
@@ -54,6 +82,10 @@ void settings_reset(GtkBuilder* gui)
   logpath=GTK_WIDGET(gtk_builder_get_object(gui, "logpath_pm"));
   gtk_entry_set_text(GTK_ENTRY(logpath), config_get_str("logpath_pm"));
   // Youtube
+#ifdef HAVE_AVFORMAT
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_embed"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("youtuberadio_embed"));
+#endif
   option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_cmd"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("youtuberadio_cmd"));
   option=GTK_WIDGET(gtk_builder_get_object(gui, "youtubecmd"));
@@ -63,9 +95,18 @@ void settings_reset(GtkBuilder* gui)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("camdownonjoin"));
   option=GTK_WIDGET(gtk_builder_get_object(gui, "autoopencams"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_set("autoopencams")?config_get_bool("autoopencams"):1);
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "disablesnapshots"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("disablesnapshots"));
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "showgreenroom"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("showgreenroom"));
   // Misc/cookies
   option=GTK_WIDGET(gtk_builder_get_object(gui, "storecookiecheckbox"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("storecookies"));
+  // Misc/userlist
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "userlist_sort_mod"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("userlist_sort_mod"));
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "userlist_sort_self"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(option), config_get_bool("userlist_sort_self"));
 }
 
 void showsettings(GtkMenuItem* item, GtkBuilder* gui)
@@ -77,10 +118,31 @@ void showsettings(GtkMenuItem* item, GtkBuilder* gui)
 
 void savesettings(GtkButton* button, GtkBuilder* gui)
 {
-  // Font
+  // Text
   GtkSpinButton* fontsize=GTK_SPIN_BUTTON(gtk_builder_get_object(gui, "fontsize"));
   config_set_double("fontsize", gtk_spin_button_get_value(fontsize));
   fontsize_set(gtk_spin_button_get_value(fontsize));
+  GtkWidget* option=GTK_WIDGET(gtk_builder_get_object(gui, "linewrap"));
+  GtkTreeModel* model=gtk_combo_box_get_model(GTK_COMBO_BOX(option));
+  GtkTreeIter iter;
+  if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(option), &iter))
+  {
+    GtkWrapMode mode;
+    gtk_tree_model_get(model, &iter, 1, &mode, -1);
+    config_set_int("linewrap_mode", mode);
+    GtkTextView* chatview=GTK_TEXT_VIEW(gtk_builder_get_object(gui, "chatview"));
+    gtk_text_view_set_wrap_mode(chatview, mode);
+    unsigned int i;
+    for(i=0; i<usercount; ++i)
+    {
+      if(!userlist[i].pm_chatview){continue;}
+      gtk_text_view_set_wrap_mode(userlist[i].pm_chatview->textview, mode);
+    }
+  }
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "bluelinks"));
+  config_set("blue_links", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "hidenotifications"));
+  config_set("hide_notifications", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
   // Sound
   GtkWidget* soundcmd=GTK_WIDGET(gtk_builder_get_object(gui, "soundcmd"));
   config_set("soundcmd", gtk_entry_get_text(GTK_ENTRY(soundcmd)));
@@ -95,18 +157,37 @@ void savesettings(GtkButton* button, GtkBuilder* gui)
   logpath=GTK_WIDGET(gtk_builder_get_object(gui, "logpath_pm"));
   config_set("logpath_pm", gtk_entry_get_text(GTK_ENTRY(logpath)));
   // Youtube
+#ifdef HAVE_AVFORMAT
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_embed"));
+  config_set("youtuberadio_embed", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+#endif
   GtkWidget* youtubecmd=GTK_WIDGET(gtk_builder_get_object(gui, "youtubecmd"));
   config_set("youtubecmd", gtk_entry_get_text(GTK_ENTRY(youtubecmd)));
-  GtkWidget* youtuberadio_cmd=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_cmd"));
-  config_set("youtuberadio_cmd", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(youtuberadio_cmd))?"True":"False");
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_cmd"));
+  config_set("youtuberadio_cmd", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
   // Cameras
-  GtkWidget* option=GTK_WIDGET(gtk_builder_get_object(gui, "camdownonjoin"));
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "camdownonjoin"));
   config_set("camdownonjoin", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
   option=GTK_WIDGET(gtk_builder_get_object(gui, "autoopencams"));
   config_set("autoopencams", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "disablesnapshots"));
+  char v=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option));
+  if(nickname && v!=config_get_bool("disablesnapshots"))
+  {
+    write(tc_client_in[1], v?"/disablesnapshots\n":"/enablesnapshots\n", v?18:17);
+  }
+  config_set("disablesnapshots", v?"True":"False");
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "showgreenroom"));
+  config_set("showgreenroom", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
   // Misc/cookies
   option=GTK_WIDGET(gtk_builder_get_object(gui, "storecookiecheckbox"));
   config_set("storecookies", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  // Misc/userlist
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "userlist_sort_mod"));
+  config_set("userlist_sort_mod", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "userlist_sort_self"));
+  config_set("userlist_sort_self", gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(option))?"True":"False");
+  userlist_sort();
 
   config_save();
   GtkWidget* settings=GTK_WIDGET(gtk_builder_get_object(gui, "settings"));
@@ -532,7 +613,12 @@ void gui_hide_cam(GtkMenuItem* menuitem, void* x)
   struct camera* cam=camera_find(menu_context_cam);
   if(!cam){return;}
   dprintf(tc_client_in[1], "/closecam %s\n", cam->nick);
-  camera_remove(menu_context_cam, 0);
+  if(!strcmp(cam->id, "out"))
+  {
+    stopbroadcasting(0, 0);
+  }else{
+    camera_remove(menu_context_cam, 0);
+  }
 }
 
 gboolean gui_greenscreen_preview(void* x)
@@ -651,7 +737,13 @@ void gui_insert_link(GtkTextBuffer* buffer, GtkTextIter* iter, const char* url,
   int startnum=gtk_text_iter_get_offset(iter);
   gtk_text_buffer_insert(buffer, iter, url, length);
   // Make it look like a link and store the URL for later
-  GtkTextTag* tag=gtk_text_buffer_create_tag(buffer, 0, "underline", PANGO_UNDERLINE_SINGLE, (char*)0);
+  GtkTextTag* tag;
+  if(config_get_bool("blue_links"))
+  {
+    tag=gtk_text_buffer_create_tag(buffer, 0, "underline", PANGO_UNDERLINE_SINGLE, "foreground", "#0000ff", (char*)0);
+  }else{
+    tag=gtk_text_buffer_create_tag(buffer, 0, "underline", PANGO_UNDERLINE_SINGLE, (char*)0);
+  }
   g_object_set_data(G_OBJECT(tag), "url", strndup(url, length));
   GtkTextIter start;
   gtk_text_buffer_get_iter_at_offset(buffer, &start, startnum);
@@ -762,7 +854,12 @@ struct chatview* chatview_new(GtkTextView* existing_textview)
   this->atbottom=1;
   gtk_text_view_set_editable(this->textview, 0);
   gtk_text_view_set_cursor_visible(this->textview, 0);
-  gtk_text_view_set_wrap_mode(this->textview, GTK_WRAP_CHAR);
+  if(config_get_set("linewrap_mode"))
+  {
+    gtk_text_view_set_wrap_mode(this->textview, config_get_int("linewrap_mode"));
+  }else{
+    gtk_text_view_set_wrap_mode(this->textview, GTK_WRAP_WORD_CHAR);
+  }
   GtkAdjustment* scroll=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(this->scrolledwindow));
   g_signal_connect(this->textview, "button-release-event", G_CALLBACK(gui_click_link), 0);
   g_signal_connect(this->textview, "button-press-event", G_CALLBACK(gui_rightclick_link), 0);
@@ -938,8 +1035,10 @@ void gui_init(char frombuild)
   // Youtube
   option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_cmd"));
   g_signal_connect(option, "toggled", G_CALLBACK(toggle_youtubecmd), gui);
-  // Misc
-  option=GTK_WIDGET(gtk_builder_get_object(gui, "camdownonjoin"));
+  #ifndef HAVE_AVFORMAT
+  option=GTK_WIDGET(gtk_builder_get_object(gui, "youtuberadio_embed"));
+  gtk_widget_destroy(option);
+  #endif
 
   GtkWidget* window=GTK_WIDGET(gtk_builder_get_object(gui, "main"));
   g_signal_connect(window, "configure-event", G_CALLBACK(handleresize), 0);
@@ -986,6 +1085,13 @@ void gui_init(char frombuild)
     camplaceholder=gdk_pixbuf_animation_new_from_file(PREFIX "/share/tc_client/camplaceholder.gif", 0);
   }
   camplaceholder_iter=gdk_pixbuf_animation_get_iter(camplaceholder, 0);
+  // Load moderator icon
+  if(frombuild)
+  {
+    modicon=gdk_pixbuf_new_from_file("modicon.png", 0);
+  }else{
+    modicon=gdk_pixbuf_new_from_file(PREFIX "/share/tc_client/modicon.png", 0);
+  }
   // Populate saved channels
   GtkWidget* startbox=GTK_WIDGET(gtk_builder_get_object(gui, "startbox"));
   int channelcount=config_get_int("channelcount");
@@ -1016,3 +1122,9 @@ void gui_init(char frombuild)
   }
   gtk_widget_show_all(startwindow);
 }
+
+void gui_disableinputs(void)
+{
+  gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(gui, "inputfield")), 0);
+  gtk_widget_set_sensitive(GTK_WIDGET(gtk_builder_get_object(gui, "menuitem_broadcast")), 0);
+}
