@@ -14,6 +14,13 @@
     You should have received a copy of the GNU Affero General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
+#define TC_CLIENT (frombuild?"./tc_client":"tc_client")
+extern int tc_client_in[2];
+extern const char* channel;
+extern char* nickname;
+extern char hasgreenroom;
+extern char frombuild;
+extern gboolean handledata(GIOChannel* iochannel, GIOCondition condition, gpointer x);
 extern void togglecam(GtkCheckMenuItem* item, void* x);
 #ifdef HAVE_PULSEAUDIO
 void togglemic(GtkCheckMenuItem* item, void* x);
