@@ -19,6 +19,7 @@
 #include <gtk/gtk.h>
 #include <libavcodec/avcodec.h>
 #include <libswscale/swscale.h>
+#include <libavutil/opt.h>
 #if LIBAVUTIL_VERSION_MAJOR>50 || (LIBAVUTIL_VERSION_MAJOR==50 && LIBAVUTIL_VERSION_MINOR>37)
   #include <libavutil/imgutils.h>
 #else
@@ -32,6 +33,8 @@
 #include "../compat.h"
 #include "../compat_av.h"
 #include "gui.h"
+#include "main.h"
+#include "greenroom.h"
 #include "media.h"
 
 #define PREVIEW_MAX_WIDTH 640
@@ -56,35 +59,48 @@ CAM* camout_cam=0;
 char pushtotalk_enabled=0;
 char pushtotalk_pushed=0;
 #endif
+unsigned int camout_delay;
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-// Experimental mixer, not sure if it really works
-void camera_playsnd(int audiopipe, struct camera* cam, short* samples, unsigned int samplecount)
+void camera_playsnd(struct camera* cam, int16_t* samples, unsigned int samplecount)
 {
-  if(cam->samples)
+  cam->samples=realloc(cam->samples, sizeof(int16_t)*(cam->samplecount+samplecount));
+  memcpy(&cam->samples[cam->samplecount], samples, samplecount*sizeof(short));
+  cam->samplecount+=samplecount;
+}
+
+gboolean audiomixer(void* p)
+{
+  int audiopipe=*(int*)p;
+  unsigned int i;
+  int sources=0;
+  for(i=0; i<camcount; ++i){sources+=!!cams[i].samplecount;}
+  if(!sources){return G_SOURCE_CONTINUE;}
+  unsigned int samplecount=SAMPLERATE_OUT/25; // Play one 25th of the samplerate's samples per iteration (which happens 25 times per second)
+  int16_t samples[samplecount];
+  memset(samples, 0, samplecount*sizeof(int16_t));
+  for(i=0; i<camcount; ++i)
   {
-// int sources=1;
-    unsigned int i;
-    for(i=0; i<camcount; ++i)
+    if(!cams[i].samplecount){continue;}
+    unsigned j;
+    for(j=0; j<samplecount && j<cams[i].samplecount; ++j)
     {
-      if(!cams[i].samples){continue;}
-      if(cam==&cams[i]){continue;}
-      unsigned j;
-      for(j=0; j<cam->samplecount && j<cams[i].samplecount; ++j)
-      {
-        cam->samples[j]+=cams[i].samples[j];
-      }
-      free(cams[i].samples);
-      cams[i].samples=0;
-// ++sources;
+      // Divide by number of sources to prevent integer overflow
+      samples[j]+=cams[i].samples[j]/sources;
     }
-    write(audiopipe, cam->samples, cam->samplecount*sizeof(short));
-    free(cam->samples);
-// printf("Mixed sound from %i sources (cam: %p)\n", sources, cam);
+    if(cams[i].samplecount>samplecount)
+    {
+      cams[i].samplecount-=samplecount;
+      // Deal with drift and post-lag floods
+      if(cams[i].samplecount>SAMPLERATE_OUT/5){cams[i].samplecount=0; continue;}
+    }else{
+      cams[i].samplecount=0;
+      continue;
+    }
+    memmove(cams[i].samples, &cams[i].samples[samplecount], sizeof(int16_t)*cams[i].samplecount);
   }
-  cam->samples=malloc(samplecount*sizeof(short));
-  memcpy(cam->samples, samples, samplecount*sizeof(short));
-  cam->samplecount=samplecount;
+  write(audiopipe, samples, samplecount*sizeof(int16_t));
+  return G_SOURCE_CONTINUE;
 }
 #endif
 
@@ -147,32 +163,47 @@ struct camera* camera_findbynick(const char* nick)
   return 0;
 }
 
-struct camera* camera_new(const char* nick, const char* id)
+struct camera* camera_new(const char* nick, const char* id, unsigned char flags)
 {
   ++camcount;
   cams=realloc(cams, sizeof(struct camera)*camcount);
   struct camera* cam=&cams[camcount-1];
+  cam->vctx=0;
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
   cam->actx=0;
   cam->samples=0;
   cam->samplecount=0;
+  #ifdef HAVE_AVRESAMPLE
+  cam->resamplectx=0;
+  #else
+  cam->swrctx=0;
+  #endif
 #endif
   cam->nick=strdup(nick);
   cam->id=strdup(id);
   cam->frame=av_frame_alloc();
   cam->dstframe=av_frame_alloc();
   cam->cam=gtk_image_new();
-  cam->box=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-  gtk_box_set_homogeneous(GTK_BOX(cam->box), 0);
-  // Wrap cam image in an event box to catch (right) clicks
-  GtkWidget* eventbox=gtk_event_box_new();
-  gtk_container_add(GTK_CONTAINER(eventbox), cam->cam);
-  gtk_event_box_set_above_child(GTK_EVENT_BOX(eventbox), 1);
-  cam->label=gtk_label_new(cam->nick);
-  gtk_box_pack_start(GTK_BOX(cam->box), eventbox, 0, 0, 0);
-  gtk_box_pack_start(GTK_BOX(cam->box), cam->label, 0, 0, 0);
-  g_signal_connect(eventbox, "button-release-event", G_CALLBACK(gui_show_cam_menu), cam->id);
-  cam->placeholder=g_timeout_add(100, camplaceholder_update, cam->id);
+  if(flags&CAMFLAG_GREENROOM)
+  {
+    cam->box=0;
+    cam->placeholder=0;
+  }else{
+    cam->box=gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+    gtk_box_set_homogeneous(GTK_BOX(cam->box), 0);
+    // Wrap cam image in an event box to catch (right) clicks
+    GtkWidget* eventbox=gtk_event_box_new();
+    gtk_container_add(GTK_CONTAINER(eventbox), cam->cam);
+    gtk_event_box_set_above_child(GTK_EVENT_BOX(eventbox), 1);
+    cam->label=gtk_label_new(cam->nick);
+    gtk_box_pack_start(GTK_BOX(cam->box), eventbox, 0, 0, 0);
+    gtk_box_pack_start(GTK_BOX(cam->box), cam->label, 0, 0, 0);
+    g_signal_connect(eventbox, "button-release-event", G_CALLBACK(gui_show_cam_menu), cam->id);
+    cam->placeholder=g_timeout_add(100, camplaceholder_update, cam->id);
+  }
+  cam->volume=0;
+  cam->volumeold=1024;
+  cam->flags=flags;
   // Initialize postprocessing values
   postproc_init(&cam->postproc);
   return cam;
@@ -230,12 +261,12 @@ void camera_cleanup(void)
 
 void freebuffer(guchar* pixels, gpointer data){free(pixels);}
 
-unsigned int camout_delay;
 void startcamout(CAM* cam)
 {
   if(!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(gtk_builder_get_object(gui, "menuitem_broadcast_mic"))))
   { // Only /camup if we're not already broadcasting mic
-    dprintf(tc_client_in[1], "/camup\n");
+    int fd=((hasgreenroom && !greenroom_gotpass)?greenroompipe_in[1]:tc_client_in[1]);
+    dprintf(fd, "/camup\n");
   }
   camout_cam=cam;
   camout_delay=500;
@@ -245,11 +276,40 @@ void startcamout(CAM* cam)
   g_timeout_add(camout_delay, cam_encode, cam);
 }
 
+GdkPixbuf* scaleframe(void* data, unsigned int width, unsigned int height, unsigned int linesize, unsigned int towidth, unsigned int toheight)
+{
+  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(data, GDK_COLORSPACE_RGB, 0, 8, width, height, linesize, 0, 0);
+  // Scale to fit
+  GdkPixbuf* scaled=gdk_pixbuf_new(GDK_COLORSPACE_RGB, 0, 8, towidth, toheight);
+  double scalex=(double)towidth/(double)width;
+  double scaley=(double)toheight/(double)height;
+  unsigned int offsetx=0;
+  unsigned int offsety=0;
+  if(height*4/3>width)
+  {
+    scalex=scaley;
+    offsetx=(towidth-width*toheight/height)/2;
+    towidth=width*toheight/height;
+    gdk_pixbuf_fill(scaled, 0);
+  }
+  else if(width*3/4>height)
+  {
+    scaley=scalex;
+    offsety=(toheight-height*towidth/width)/2;
+    toheight=height*towidth/width;
+    gdk_pixbuf_fill(scaled, 0);
+  }
+  gdk_pixbuf_scale(gdkframe, scaled, offsetx, offsety, towidth, toheight, offsetx, offsety, scalex, scaley, GDK_INTERP_BILINEAR);
+  g_object_unref(gdkframe);
+  return scaled;
+}
+
 gboolean cam_encode(void* camera_)
 {
   CAM* camera=camera_;
   if(camera!=camout_cam){return G_SOURCE_REMOVE;}
   struct camera* cam=camera_find("out");
+  if(!cam){return G_SOURCE_REMOVE;}
   if(cam->placeholder) // Remove the placeholder animation if it has it
   {
     g_source_remove(cam->placeholder);
@@ -277,9 +337,16 @@ gboolean cam_encode(void* camera_)
   postprocess(&cam->postproc, cam->frame->data[0], cam->frame->width, cam->frame->height);
   // Update our local display
   GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
-  GdkPixbuf* gdkframe=gdk_pixbuf_new_from_data(cam->frame->data[0], GDK_COLORSPACE_RGB, 0, 8, cam->frame->width, cam->frame->height, cam->frame->linesize[0], 0, 0);
-  // Scale to fit
-  gdkframe=gdk_pixbuf_scale_simple(gdkframe, camsize_scale.width, camsize_scale.height, GDK_INTERP_BILINEAR);
+  GdkPixbuf* gdkframe=scaleframe(cam->frame->data[0], cam->frame->width, cam->frame->height, cam->frame->linesize[0], camsize_scale.width, camsize_scale.height);
+  volume_indicator(gdkframe, cam); // Add volume indicator
+  int fd;
+  if(hasgreenroom && !greenroom_gotpass)
+  {
+    fd=greenroompipe_in[1];
+    greenroom_indicator(gdkframe); // Add greenroom indicator
+  }else{
+    fd=tc_client_in[1];
+  }
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
   g_object_unref(oldpixbuf);
   // Encode
@@ -300,11 +367,12 @@ gboolean cam_encode(void* camera_)
   avcodec_send_frame(cam->vctx, cam->dstframe);
   gotpacket=avcodec_receive_packet(cam->vctx, &packet);
   if(gotpacket){return G_SOURCE_CONTINUE;}
-  unsigned char frameinfo=0x22; // Note: differentiating between keyframes and non-keyframes seems to break stuff, so let's just go with all being interframes (1=keyframe, 2=interframe, 3=disposable interframe)
+  char key=!!(packet.flags&AV_PKT_FLAG_KEY);
+  unsigned char frameinfo=(key?0x12:0x22); // In the first 4 bits: 1=keyframe, 2=interframe
   // Send video
-  dprintf(tc_client_in[1], "/video %i\n", packet.size+1);
-  write(tc_client_in[1], &frameinfo, 1);
-  ssize_t w=write(tc_client_in[1], packet.data, packet.size);
+  dprintf(fd, "/video %i\n", packet.size+1);
+  write(fd, &frameinfo, 1);
+  ssize_t w=write(fd, packet.data, packet.size);
 if(w!=packet.size){printf("Error: wrote %zi of %i bytes\n", w, packet.size);}
 
   av_packet_unref(&packet);
@@ -356,7 +424,7 @@ void camselect_open(void(*cb)(CAM*), void(*ccb)(void))
 void camselect_change(GtkComboBox* combo, void* x)
 {
   if(!camselect.eventsource){return;} // Haven't opened the cam selection window yet (happens at startup)
-  if(camselect.current){cam_close(camselect.current);}
+  if(camselect.current){cam_close(camselect.current); camselect.current=0;}
   camselect.current=cam_open(gtk_combo_box_get_active_id(combo));
   if(!camselect.current){return;}
   camselect.size.width=320;
@@ -408,11 +476,7 @@ void camselect_file_preview(GtkFileChooser* dialog, gpointer data)
 const char* camselect_file(void)
 {
   GtkWidget* preview=gtk_image_new();
-#ifndef _WIN32
   GtkWindow* window=GTK_WINDOW(gtk_builder_get_object(gui, "camselection"));
-#else
-  GtkWindow* window=0;
-#endif
   GtkWidget* dialog=gtk_file_chooser_dialog_new("Open Image", window, GTK_FILE_CHOOSER_ACTION_OPEN, "_Cancel", GTK_RESPONSE_CANCEL, "_Open", GTK_RESPONSE_ACCEPT, (char*)0);
   GtkFileFilter* filter=gtk_file_filter_new();
   gtk_file_filter_add_pixbuf_formats(filter);
@@ -431,7 +495,13 @@ const char* camselect_file(void)
 
 void updatescaling(unsigned int width, unsigned int height, char changedcams)
 {
-  if(!camcount){return;}
+  unsigned int boxcount=0;
+  unsigned int i;
+  for(i=0; i<camcount; ++i)
+  {
+    if(!cams[i].flags&CAMFLAG_GREENROOM){++boxcount;}
+  }
+  if(!boxcount){return;}
   if(!width){width=gtk_widget_get_allocated_width(GTK_WIDGET(gtk_builder_get_object(gui, "main")));}
   if(!height){height=gtk_widget_get_allocated_height(GTK_WIDGET(gtk_builder_get_object(gui, "camerascroll")));}
 
@@ -441,11 +511,11 @@ void updatescaling(unsigned int width, unsigned int height, char changedcams)
   camsize_scale.height=1;
   unsigned int rowcount=1;
   unsigned int rows;
-  for(rows=1; rows<=camcount; ++rows)
+  for(rows=1; rows<=boxcount; ++rows)
   {
     struct size scale;
-    unsigned int cams_per_row=camcount/rows;
-    if(camcount%rows){++cams_per_row;}
+    unsigned int cams_per_row=boxcount/rows;
+    if(boxcount%rows){++cams_per_row;}
     scale.width=width/cams_per_row;
     // 3/4 ratio
     scale.height=scale.width*3/4;
@@ -464,11 +534,11 @@ void updatescaling(unsigned int width, unsigned int height, char changedcams)
     }else if(scale.width<camsize_scale.width){break;} // Only getting smaller from here, use the last one that increased
   }
 
-  unsigned int i;
   if(rowcount!=camrowcount || changedcams) // Changed the number of rows, shuffle everything around to fit. Or added/removed a camera, in which case we need to shuffle things around anyway
   {
     for(i=0; i<camcount; ++i)
     {
+      if(cams[i].flags&CAMFLAG_GREENROOM){continue;}
       g_object_ref(cams[i].box); // Increase reference counts so that they are not deallocated while they are temporarily detached from the rows
       GtkContainer* parent=GTK_CONTAINER(gtk_widget_get_parent(cams[i].box));
       if(parent){gtk_container_remove(parent, cams[i].box);}
@@ -483,11 +553,14 @@ void updatescaling(unsigned int width, unsigned int height, char changedcams)
       gtk_widget_set_halign(camrows[i], GTK_ALIGN_CENTER);
       gtk_widget_show(camrows[i]);
     }
-    unsigned int cams_per_row=camcount/camrowcount;
-    if(camcount%camrowcount){++cams_per_row;}
+    unsigned int cams_per_row=boxcount/camrowcount;
+    if(boxcount%camrowcount){++cams_per_row;}
+    unsigned int index=0;
     for(i=0; i<camcount; ++i)
     {
-      gtk_box_pack_start(GTK_BOX(camrows[i/cams_per_row]), cams[i].box, 0, 0, 0);
+      if(cams[i].flags&CAMFLAG_GREENROOM){continue;}
+      gtk_box_pack_start(GTK_BOX(camrows[index/cams_per_row]), cams[i].box, 0, 0, 0);
+      ++index;
       g_object_unref(cams[i].box); // Decrease reference counts once they're attached again
     }
   }
@@ -497,6 +570,7 @@ void updatescaling(unsigned int width, unsigned int height, char changedcams)
   // Rescale current images to fit
   for(i=0; i<camcount; ++i)
   {
+    if(cams[i].flags&CAMFLAG_GREENROOM){continue;}
     GdkPixbuf* pixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cams[i].cam));
     if(!pixbuf){continue;}
     GdkPixbuf* old=pixbuf;
@@ -515,6 +589,7 @@ gboolean camplaceholder_update(void* id)
   GdkPixbuf* frame=gdk_pixbuf_animation_iter_get_pixbuf(camplaceholder_iter);
   // Scale and replace the current image on camera
   GdkPixbuf* pixbuf=gdk_pixbuf_scale_simple(frame, camsize_scale.width, camsize_scale.height, GDK_INTERP_BILINEAR);
+  volume_indicator(pixbuf, cam); // Add volume indicator
   gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), pixbuf);
   if(oldpixbuf){g_object_unref(oldpixbuf);}
   return G_SOURCE_CONTINUE;
@@ -540,6 +615,7 @@ GdkPixbuf* scaled_gdk_pixbuf_from_cam(CAM* cam, unsigned int width, unsigned int
 }
 
 #ifdef HAVE_PULSEAUDIO
+extern void justwait(int);
 void* audiothread_in(void* fdp)
 {
   int fd=*(int*)fdp;
@@ -548,7 +624,10 @@ void* audiothread_in(void* fdp)
   pulsespec.format=PA_SAMPLE_FLOAT32;
   pulsespec.channels=1;
   pulsespec.rate=44100;
+  signal(SIGCHLD, SIG_DFL); // Briefly revert to the default handler to not break pulseaudio's autospawn feature
   pulse=pa_simple_new(0, "tc_client-gtk", PA_STREAM_RECORD, 0, "mic", &pulsespec, 0, 0, 0);
+  signal(SIGCHLD, justwait);
+  if(!pulse){return 0;}
   char buf[1024];
   // Just read/listen and write to the main thread until either reading/listening or writing fails
   while(1)
@@ -590,8 +669,18 @@ gboolean mic_encode(GIOChannel* iochannel, GIOCondition condition, gpointer data
   gsize r;
   int status=g_io_channel_read_chars(iochannel, (char*)buf, 1024, &r, 0);
   if(status!=G_IO_STATUS_NORMAL){return 0;}
+
+  struct camera* cam=camera_find("out");
   // If push-to-talk is enabled but not pushed, return as soon as we've consumed the incoming data
-  if(pushtotalk_enabled && !pushtotalk_pushed){return 1;}
+  if(pushtotalk_enabled && !pushtotalk_pushed)
+  {
+    if(cam){cam->volume=0;}
+    return 1;
+  }
+  if(cam)
+  {
+    camera_calcvolume(cam, (float*)buf, r/sizeof(float));
+  }
   micframe->nb_samples=r/4; // 32bit floats
   micframe->data[0]=buf;
   micframe->linesize[0]=r;
@@ -609,11 +698,116 @@ gboolean mic_encode(GIOChannel* iochannel, GIOCondition condition, gpointer data
   if(avcodec_receive_packet(avctx, &packet)){return 1;}
   unsigned char frameinfo=0x6c; // 6=Nellymoser, 3<<2=44100 samplerate
   // Send audio
-  dprintf(tc_client_in[1], "/audio %i\n", packet.size+1);
-  write(tc_client_in[1], &frameinfo, 1);
-  write(tc_client_in[1], packet.data, packet.size);
+  int fd=((hasgreenroom && !greenroom_gotpass)?greenroompipe_in[1]:tc_client_in[1]);
+  dprintf(fd, "/audio %i\n", packet.size+1);
+  write(fd, &frameinfo, 1);
+  write(fd, packet.data, packet.size);
 
   av_packet_unref(&packet);
   return 1;
 }
 #endif
+
+void camera_calcvolume(struct camera* cam, float* samples, unsigned int samplecount)
+{
+  unsigned int i;
+  float max=0;
+  float min=99999;
+  for(i=0; i<samplecount; ++i)
+  {
+    if(samples[i]>max){max=samples[i];}
+    if(samples[i]<min){min=samples[i];}
+  }
+  float diff=max-min;
+  if(diff>cam->volume || cam->volumeold){cam->volume=diff; cam->volumeold=0;}
+}
+
+void volume_indicator(GdkPixbuf* frame, struct camera* cam)
+{
+  if(!frame){return;}
+  if(cam->volumeold>10){return;} // Not sending any audio anymore
+  guchar* pixels=gdk_pixbuf_get_pixels(frame);
+  unsigned int channels=gdk_pixbuf_get_n_channels(frame);
+  unsigned int stride=gdk_pixbuf_get_rowstride(frame);
+  unsigned int width=gdk_pixbuf_get_width(frame);
+  unsigned int height=gdk_pixbuf_get_height(frame);
+  unsigned int size_x=width/24;
+  unsigned int size_y=height/5;
+  unsigned int pos_x=width*47/48-size_x;
+  unsigned int pos_y=height*47/48-size_y;
+  int volumebar=size_y-cam->volume*size_y;
+  if(volumebar<0){volumebar=0;}
+  unsigned int x, y;
+  for(y=0; y<size_y; ++y)
+  {
+    for(x=0; x<size_x; ++x)
+    {
+      if(y>=volumebar && x>=size_x/8 && x<size_x-size_x/8)
+      { // Green bar
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels]=0x0;
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels+1]=0xff;
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels+2]=0x0;
+      }else{ // Gray background
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels]=0x80;
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels+1]=0x80;
+        pixels[stride*(y+pos_y)+(x+pos_x)*channels+2]=0x80;
+      }
+    }
+  }
+  ++cam->volumeold;
+}
+
+void camera_decode(struct camera* cam, AVPacket* pkt, unsigned int width, unsigned int height)
+{
+  if(!cam->vctx)
+  {
+    AVCodec* codec=avcodec_find_decoder(AV_CODEC_ID_FLV1);
+    cam->vctx=avcodec_alloc_context3(codec);
+    avcodec_open2(cam->vctx, codec, 0);
+  }
+  int gotframe;
+  avcodec_send_packet(cam->vctx, pkt);
+  gotframe=avcodec_receive_frame(cam->vctx, cam->frame);
+  if(gotframe){return;}
+
+  if(cam->placeholder) // Remove the placeholder animation if it has it
+  {
+    g_source_remove(cam->placeholder);
+    cam->placeholder=0;
+  }
+  // Scale and convert to RGB24 format
+  unsigned int bufsize=av_image_get_buffer_size(AV_PIX_FMT_RGB24, cam->frame->width, cam->frame->height, 1);
+  unsigned char buf[bufsize];
+  cam->dstframe->data[0]=buf;
+  cam->dstframe->linesize[0]=cam->frame->width*3;
+  struct SwsContext* swsctx=sws_getContext(cam->frame->width, cam->frame->height, cam->frame->format, cam->frame->width, cam->frame->height, AV_PIX_FMT_RGB24, SWS_BICUBIC, 0, 0, 0);
+  sws_scale(swsctx, (const uint8_t*const*)cam->frame->data, cam->frame->linesize, 0, cam->frame->height, cam->dstframe->data, cam->dstframe->linesize);
+  sws_freeContext(swsctx);
+  postprocess(&cam->postproc, cam->dstframe->data[0], cam->frame->width, cam->frame->height);
+
+  GdkPixbuf* oldpixbuf=gtk_image_get_pixbuf(GTK_IMAGE(cam->cam));
+  GdkPixbuf* gdkframe=scaleframe(cam->dstframe->data[0], cam->frame->width, cam->frame->height, cam->dstframe->linesize[0], width, height);
+  volume_indicator(gdkframe, cam);
+  gtk_image_set_from_pixbuf(GTK_IMAGE(cam->cam), gdkframe);
+  if(oldpixbuf){g_object_unref(oldpixbuf);}
+}
+
+#ifdef HAVE_LIBAO
+void mic_decode(struct camera* cam, AVPacket* pkt)
+{
+  int gotframe;
+  avcodec_send_packet(cam->actx, pkt);
+  gotframe=avcodec_receive_frame(cam->actx, cam->frame);
+  if(gotframe){return;}
+  camera_calcvolume(cam, (float*)cam->frame->data[0], cam->frame->nb_samples);
+  unsigned int samplecount=cam->frame->nb_samples*SAMPLERATE_OUT/cam->samplerate;
+  int16_t outbuf[samplecount];
+  void* outdata[]={outbuf, 0};
+#ifdef HAVE_AVRESAMPLE
+  int outlen=avresample_convert(cam->resamplectx, (void*)outdata, samplecount*sizeof(uint8_t), samplecount, cam->frame->data, cam->frame->linesize[0], cam->frame->nb_samples);
+#else
+  int outlen=swr_convert(cam->swrctx, (void*)outdata, samplecount, (const uint8_t**)cam->frame->data, cam->frame->nb_samples);
+#endif
+  if(outlen>0){camera_playsnd(cam, outbuf, outlen);}
+}
+#endif
