@@ -16,6 +16,8 @@
 */
 #ifndef MEDIA_H
 #define MEDIA_H
+#include <glib.h>
+#include <gtk/gtk.h>
 #include <libavcodec/avcodec.h>
 #ifdef HAVE_AVRESAMPLE
   #include <libavresample/avresample.h>
@@ -25,6 +27,8 @@
 #include "../libcamera/camera.h"
 #include "postproc.h"
 #define SAMPLERATE_OUT 11025 // 11025 is the most common input sample rate, and it is more CPU-efficient to keep it that way than upsampling or downsampling it when converting from flt to s16
+#define CAMFLAG_NONE      0
+#define CAMFLAG_GREENROOM 1
 struct camera
 {
   AVFrame* frame;
@@ -46,6 +50,9 @@ struct camera
   SwrContext* swrctx;
 #endif
   unsigned int samplerate;
+  float volume;
+  unsigned int volumeold;
+  unsigned char flags;
 };
 struct size
 {
@@ -63,14 +70,16 @@ extern GdkPixbufAnimationIter* camplaceholder_iter;
 extern CAM* camout_cam;
 extern char pushtotalk_enabled;
 extern char pushtotalk_pushed;
+extern unsigned int camout_delay;
 
 #if defined(HAVE_AVRESAMPLE) || defined(HAVE_SWRESAMPLE)
-extern void camera_playsnd(int audiopipe, struct camera* cam, short* samples, unsigned int samplecount);
+extern void camera_playsnd(struct camera* cam, int16_t* samples, unsigned int samplecount);
+extern gboolean audiomixer(void* p);
 #endif
 extern void camera_remove(const char* id, char isnick);
 extern struct camera* camera_find(const char* id);
 extern struct camera* camera_findbynick(const char* nick);
-extern struct camera* camera_new(const char* nick, const char* id);
+extern struct camera* camera_new(const char* nick, const char* id, unsigned char flags);
 extern char camera_init_audio(struct camera* cam, uint8_t frameinfo);
 extern void camera_cleanup(void);
 extern void freebuffer(guchar* pixels, gpointer data);
@@ -87,4 +96,10 @@ extern gboolean camplaceholder_update(void* id);
 extern GdkPixbuf* scaled_gdk_pixbuf_from_cam(CAM* cam, unsigned int width, unsigned int height, unsigned int maxwidth, unsigned int maxheight);
 extern void* audiothread_in(void* fdp);
 extern gboolean mic_encode(GIOChannel* iochannel, GIOCondition condition, gpointer datap);
+extern void camera_calcvolume(struct camera* cam, float* samples, unsigned int samplecount);
+extern void volume_indicator(GdkPixbuf* frame, struct camera* cam);
+extern void camera_decode(struct camera* cam, AVPacket* pkt, unsigned int width, unsigned int height);
+#ifdef HAVE_LIBAO
+extern void mic_decode(struct camera* cam, AVPacket* pkt);
+#endif
 #endif
