@@ -18,6 +18,7 @@ struct user
 {
   char* nick;
   GtkWidget* label;
+  GtkWidget* item;
   char ismod;
   GtkWidget* pm_tab;
   GtkWidget* pm_tablabel;
@@ -34,3 +35,6 @@ extern struct user* user_find_by_tab(GtkWidget* tab);
 extern struct user* adduser(const char* nick);
 extern void renameuser(const char* old, const char* newnick);
 extern void removeuser(const char* nick);
+extern void usersetmod(const char* nick, char mod);
+extern void userlist_sort(void);
+extern char user_ismod(const char* nick);
