@@ -1,6 +1,6 @@
 /*
     libcamera, a camera access abstraction library
-    Copyright (C) 2015  alicia@ion.nu
+    Copyright (C) 2015-2016  alicia@ion.nu
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as published by
@@ -14,10 +14,13 @@
     You should have received a copy of the GNU Affero General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
+#ifndef CAMERA_H
+#define CAMERA_H
 enum{
   CAMTYPE_V4L2,
   CAMTYPE_ESCAPI,
-  CAMTYPE_IMG
+  CAMTYPE_IMG,
+  CAMTYPE_X11
 };
 struct CAM_t;
 typedef struct CAM_t CAM;
@@ -28,3 +31,4 @@ extern void cam_resolution(CAM* cam, unsigned int* width, unsigned int* height);
 extern void cam_getframe(CAM* cam, void* pixmap);
 extern void cam_close(CAM* cam);
 extern const char*(*cam_img_filepicker)(void);
+#endif
